package lib;

@:structInit class Vec2<T:Float> {
	public var x:T;
	public var y:T;

	public function new(x:T, y:T) {
		this.x = x;
		this.y = y;
	}

	public function add(rhs:Vec2<T>):Vec2<T> {
		return {
			y: this.y + rhs.y,
			x: this.x + rhs.x,
		};
	}

	public function sub(rhs:Vec2<T>):Vec2<T> {
		return {
			y: this.y - rhs.y,
			x: this.x - rhs.x,
		};
	}

	public function multiply(rhs:Vec2<T>):Vec2<T> {
		return {
			y: this.y * rhs.y,
			x: this.x * rhs.x,
		};
	}

	public function negate():Vec2<T> {
		return {
			y: -this.y,
			x: -this.x,
		};
	}
}

@:structInit class Vec3<T:Float> {
	public final x:T;
	public final y:T;
	public final z:T;

	public function new(x:T, y:T, z:T) {
		this.x = x;
		this.y = y;
		this.z = z;
	}

	public function add(rhs:Vec3<T>):Vec3<T> {
		return {
			x: this.x + rhs.x,
			y: this.y + rhs.y,
			z: this.z + rhs.z
		};
	}

	public function sub(rhs:Vec3<T>):Vec3<T> {
		return {
			x: this.x - rhs.x,
			y: this.y - rhs.y,
			z: this.z - rhs.z
		};
	}

	public function multiplyScalar(rhs:T):Vec3<T> {
		return {
			x: this.x * rhs,
			y: this.y * rhs,
			z: this.z * rhs
		};
	}

	public function negate():Vec3<T> {
		return {
			x: -this.x,
			y: -this.y,
			z: -this.z
		};
	}

	public function dot(rhs:Vec3<T>):Float {
		return this.x * rhs.x + this.y * rhs.y + this.z * rhs.z;
	}

	public function cross(rhs:Vec3<T>):Vec3<T> {
		return {
			x: this.y * rhs.z - this.z * rhs.y,
			y: this.z * rhs.x - this.x * rhs.z,
			z: this.x * rhs.y - this.y * rhs.x
		};
	}

	public function length():Float {
		return Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);
	}

	public function distance(rhs:Vec3<T>):Float {
		return Math.sqrt(Math.pow(this.x - rhs.x, 2) + Math.pow(this.y - rhs.y, 2) + Math.pow(this.z - rhs.z, 2));
	}
}
